--[[
SimpleMD, an addon by Starinnia

This addon started as a simple text-warning system for use in my guild.
I released it to wowace and Curse in the hopes it would help other hunters.

Cooldown bars were recently requested and implemented.
Rather than re-invent the wheel, I looked at other addons for a useful bar implementation,
MendWatch by Polima was the source of the bars used in this addon.
]]

-- if player class is not hunter do not load anything
if (select(2, UnitClass("player"))) ~= "HUNTER" then return end

local L = AceLibrary("AceLocale-2.2"):new("SimpleMD")
local PlayerName = UnitName("player")
-- to be used soon because the hunters in my guild are lazy and I forsee version checks being needed
local revision = "$Revision: 45666 $"

SimpleMD = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceComm-2.0", "Sink-1.0", "CandyBar-2.0")
local DD = AceLibrary("Dewdrop-2.0")

function SimpleMD:OnInitialize()
	local colors = AceLibrary("PaintChips-2.0")
	colors:RegisterColor("gray", 0.5, 0.5, 0.5)
	colors:RegisterColor("dgreen", 0, 0.5, 0)
	
	local paintbucket = {
		"white",
		"black",
		"blue",
		"green",
		"gray",
		"dgreen",
		"yellow",
		"orange",
	}
	
	self.textures = {
		["default"] = nil,
	}
	
	local defaults = {
		messageGain = L["<player> cast Misdirection on <target>"],
		messageFade = L["Misdirection fades from <target>"],
		r = 1.0,
		g = 1.0,
		b = 1.0,
		channel = "",
		showInChannel = false,
		showInChat = false,
		showOnScreen = true,
		showFloating = false,
		showSelf = true,
		showRW = false,
		showBars = true,
		ready = true,
		ten = true,
		thirty = true,
		sixty = false,
		texture = "default",
		color = "green",
		bg = nil,
		bg_alpha = nil,
		textColor = nil,
		textSize = nil,
		barScale = nil,
		barWidth = nil,
		barHeight = nil,
		growUp = false,
		reverse = false,
		position = {},
	}
	
	local opts = {
		type='group',
		args = {
			anchor = {
				type = 'execute',
				name = L["Anchor"],
				desc = L["Show the dragable anchor."],
				func = "ToggleAnchor"
			},
			bars = {
				type = 'group',
				name = L["Bar"],
				desc = L["CandyBar Options"],
				args = {
					color = {
						type = 'text',
						name = L["Bar Color"],
						desc = L["Set the default bar color."],
						get = function () return self.db.profile.color end,
						set = function (v) self.db.profile.color = v end,
						validate = paintbucket,
					},
					bgcolor = {
						type = 'text',
						name = L["Background Color"],
						desc = L["Set the bar background color."],
						get = function () return self.db.profile.bg end,
						set = function (v) self.db.profile.bg = v end,
						validate = paintbucket,
					},
					bgalpha = {
						type = 'range',
						name = L["Background Alpha"],
						desc = L["Set the bar background alpha."],
						get = function () return self.db.profile.bg_alpha end,
						set = function (v) self.db.profile.bg_alpha = v end,
						min = 0.0,
						max = 1.0,
					},
					textcolor = {
						type = 'text',
						name = L["Text Color"],
						desc = L["Set the bar text color."],
						get = function () return self.db.profile.textColor end,
						set = function (v) self.db.profile.textColor = v end,
						validate = paintbucket,
					},
					textsize = {
						type = 'range',
						name = L["Text Size"],
						desc = L["Set the bar text size."],
						get = function () return self.db.profile.textSize end,
						set = function (v) self.db.profile.textSize = v end,
						min = 8,
						max = 20,
					},
					scale = {
						type = 'range',
						name = L["Bar Scale"],
						desc = L["Set the bar scale."],
						get = function () return self.db.profile.barScale end,
						set = function (v) self.db.profile.barScale = v end,
						min = 0.5,
						max = 1.5,
					},
					height = {
						type = 'range',
						name = L["Bar Height"],
						desc = L["Set the bar height."],
						get = function () return self.db.profile.barHeight end,
						set = function (v) self.db.profile.barHeight = v end,
						min = 8,
						max = 30,
					},
					width = {
						type = 'range',
						name = L["Bar Width"],
						desc = L["Set the bar width."],
						get = function () return self.db.profile.barWidth end,
						set = function (v) self.db.profile.barWidth = v end,
						min = 50,
						max = 300,
					},
					growth = {
						type = 'toggle',
						name = L["Grow Up"],
						desc = L["Toggle the bar growing upwards or downwards."],
						get = function () return self.db.profile.growUp end,
						set = function () self.db.profile.growUp = not self.db.profile.growUp
						self:SetCandyBarGroupGrowth("MendWatch", self.db.profile.growUp) end,
					},
					reverse = {
						type = 'toggle',
						name = L["Reverse"],
						desc = L["Toggle the bar fill direction."],
						get = function() return self.db.profile.reverse end,
						set = function(v) self.db.profile.reverse = v end,
					},
				},
			},
			messages = {
				type = 'group',
				name = L["Message Text Options"],
				desc = L["Options for setting the various alert texts"],
				args = {
					gain = {
						type = 'text',
						name = L["Gain Message"],
						desc = L["Set Misdirection Gained Message: use <player> to autofill your name and <target> to autofill MD's target's name"],
						usage = L["<your message>"],
						get = function() return self.db.profile.messageGain end,
						set = function(v) self.db.profile.messageGain = v end,
			        },
					fade = {
						type = 'text',
						name = L["Faded Message"],
						desc = L["Set Misdirection Faded Message: use <player> to autofill your name and <target> to autofill MD's target's name"],
						usage = L["<your message>"],
						get = function() return self.db.profile.messageFade end,
						set = function(v) self.db.profile.messageGain = v end,
					},
				},
			},
			color = {
				type = 'color',
				name = L["Text Color"],
				desc = L["Color for the On Screen Display"],
				get = function() return self.db.profile.r, self.db.profile.g, self.db.profile.b end,
				set = function(r, b, g) self.db.profile.r, self.db.profile.g, self.db.profile.b = r, g, b end,
			},
			broadcast = {
				type = 'group',
				name = L["Broadcast Options"],
				desc = L["Options for how and where the alerts are displayed"],
				args = {
					screen = {
						type = 'toggle',
						name = L["Show on Screen"],
						desc = L["Toggles the display of the alert message on screen"],
						get = function() return self.db.profile.showOnScreen end,
						set = function() self.db.profile.showOnScreen = not self.db.profile.showOnScreen end,
					},
					console = {
						type = 'toggle',
						name = L["Show in Chat"],
						desc = L["Toggles the display of the alerts as self messages"],
						get = function() return self.db.profile.showInChat end,
						set = function() self.db.profile.showInChat = not self.db.profile.showInchat end,
					},
					channel = {
						type = 'toggle',
						name = L["Show in Channel"],
						desc = L["Toggles broadcasting of alerts to a user defined channel"],
						get = function() return self.db.profile.showInChannel end,
						set = function() self.db.profile.showInChannel = not self.db.profile.showInChannel end,
					},
					floating = {
						type = 'toggle',
						name = L["Show in Combat Text"],
						desc = L["Toggles showing alerts in popular SCT like mods"],
						get = function() return self.db.profile.showFloating end,
						set = function() self.db.profile.showFloating = not self.db.profile.showFloating end,
					},
					showSelf = {
						type = 'toggle',
						name = L["Show Own Alerts"],
						desc = L["Toggles showing your alerts on your client"],
						get = function() return self.db.profile.showSelf end,
						set = function() self.db.profile.showSelf = not self.db.profile.showSelf end,
					},
					channelName = {
						type = 'text',
						name = L["Custom Channel"],
						desc = L["Name of custom channel"],
						usage = L["<channel name>"],
						get = function() return self.db.profile.channel end,
						set = function(v) self.db.profile.channel = v end,
					},
					raidwarning = {
						type = 'toggle',
						name = L["Show in Raid Warning"],
						desc = L["Broadcast Misdirection casts to Raid Warning"],
						get = function() return self.db.profile.showRW end,
						set = function() self.db.profile.showRW = not self.db.profile.showRW end,
					},
				},
			},
			cooldowns = {
				type = 'group',
				name = L["Cooldown Broadcast Options"],
				desc = L["Toggle which cooldown warnings you want to display to other users"],
				args = {
					bars = {
						type = 'toggle',
						name = L["Show Cooldown Bars"],
						desc = L["Toggles the creation of cooldown bars"],
						get = function() return self.db.profile.showBars end,
						set = "DisableBars",
					},
					ready = {
						type = 'toggle',
						name = L["CD Finished Warning"],
						desc = L["Display a warning when MD cooldown finishes"],
						get = function() return self.db.profile.ready end,
						set = function() self.db.profile.ready = not self.db.profile.ready end,
					},
					ten = {
						type = 'toggle',
						name = L["10 Second Warning"],
						desc = L["Display a warning with 10 seconds left on MD cooldown"],
						get = function() return self.db.profile.ten end,
						set = function() self.db.profile.ten = not self.db.profile.ten end,
					},
					thirty = {
						type = 'toggle',
						name = L["30 Second Warning"],
						desc = L["Display a warning with 30 seconds left on MD cooldown"],
						get = function() return self.db.profile.thirty end,
						set = function() self.db.profile.thirty = not self.db.profile.thirty end,
					},
					sixty = {
						type = 'toggle',
						name = L["60 Second Warning"],
						desc = L["Display a warning with 60 seconds left on MD cooldown"],
						get = function() return self.db.profile.sixty end,
						set = function() self.db.profile.sixty = not self.db.profile.sixty end,
					},
				},
			},
			test = {
				type = 'execute',
				name = L["Test Message Settings"],
				desc = L["Prints a test message(s) to the screen, chat frame, and/or channel"],
				func = "messageTest",
			},
		},
	}
	
	opts.args.config = {
		type = "execute",
		name = L["Config"],
		desc = L["Show the GUI configuration menu."],
		guiHidden = true,
		func = function() DD:Open(UIParent, 'children', function() DD:FeedAceOptionsTable(opts) end, 'cursorX', true, 'cursorY', true) end,
	}
	
	SimpleMD:RegisterDB("SimplemdDB")
	SimpleMD:RegisterDefaults('profile', defaults)
	SimpleMD:RegisterChatCommand(L["Slash-Commands"], opts)
	SimpleMD:SetCommPrefix("SimpleMD")
	
	self.anchor = self:CreateAnchor("SimpleMD", 0, 1, 0)
	
	self:RegisterCandyBarGroup("SimpleMD")
	self:SetCandyBarGroupPoint("SimpleMD", "TOP", self.anchor, "BOTTOM", 0, 0)
	self:SetCandyBarGroupGrowth("SimpleMD", self.db.profile.growUp)
end

function SimpleMD:OnEnable()
	self.tex = self:GetTexture(L["Misdirection"])
	
	self.bars = {}
	for i = 1, 20 do
		self.bars[i] = {} 
	end
	
	self:RegisterComm("SimpleMD", "GROUP", "OnCommReceive")
	
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD")
end

function SimpleMD:OnDisable()
	if self.bars then
		for i = 1, 20 do
			if self.bars[i].id then
				self:StopCandyBar(self.bars[i].id)
			end
		end
		self.bars = nil
	end
	self:UnregisrerAllEvents()
end

function SimpleMD:OnCommReceive(prefix, sender, distribution, message)	
	if self.db.profile.showInChat and message ~= "GAIN" and message ~= "READINESS" then
		self:Print(message)
	end
	if self.db.profile.showOnScreen and message ~= "GAIN" and message ~= "READINESS" then
		UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	end
	if self.db.profile.showFloating and message ~= "GAIN" and message ~= "READINESS" then
		self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
	end
	if message == "GAIN" and self.db.profile.showBars then
		self:StartTimer(sender)
	end
	if message == "READINESS" and self.db.profile.showBars then
		local idnum = sender.."1"
		self:StopCandyBar(idnum)
	end
end

function SimpleMD:SendMessage(message, id)	
	SimpleMD:SendCommMessage("GROUP", message)
	
	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChannel and self.db.profile.channel ~= "" then
			SendChatMessage(message, "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
		end
	
		if self.db.profile.showFloating and self.db.profile.showSelf and id ~= 1 then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
	end
end

--check to see if spell is Misdirection, if it is set the spell target
function SimpleMD:UNIT_SPELLCAST_SENT(arg1, arg2, arg3, arg4)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self.SpellTarget = arg4
		end
	end
end

-- if spellcast succeeds call the gain function and schedule cooldown events
-- needed to make sure messages are not sent when target is out of LOS or out of range or invalid
function SimpleMD:UNIT_SPELLCAST_SUCCEEDED(arg1, arg2, arg3)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self:Gain()
			if self.db.profile.sixty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 60, 60) --60 second warning
			end
			if self.db.profile.thirty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 90, 30) --30 second warning
			end
			if self.db.profile.ten then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 110, 10) -- 10 second warning
			end
			if self.db.profile.ready then
			self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD", 120, 0) -- cooldown finished
			end
		end
	end
	
	--account for the readiness talent
	if arg1 == "player" and arg2 == L["Readiness"] then
		self:CancelAllScheduledEvents()
		local temp = PlayerName .. " " .. L["used Readiness! MD ready!"]
		self:SendMessage(temp)
		SimpleMD:SendCommMessage("GROUP", "READINESS")
		if self.db.profile.showSelf and self.db.profile.showBars then
		local idnum = sender.."1"
			self:StopCandyBar(idnum)
		end
	end
end

function SimpleMD:CHAT_MSG_SPELL_AURA_GONE_SELF(arg1)
	if arg1 == L["Misdirection fades from you."] then
		self:Fade()
	end
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD(arg1)
	if arg1 == 60 then
		local temp = PlayerName .. " " .. L["has 60 seconds left on MD cooldown."]
		self:SendMessage(temp, 2)
	end
	if arg1 == 30 then
		local temp = PlayerName .. " " .. L["has 30 seconds left on MD cooldown."]
		self:SendMessage(temp, 2)
	end
	if arg1 == 10 then
		local temp = PlayerName .. " " .. L["has 10 seconds left on MD cooldown!"]
		self:SendMessage(temp, 2)
	end
	if arg1 == 0 then
		local temp = PlayerName .. " " .. L["has MD ready!"]
		self:SendMessage(temp, 2)
	end
end

-- called on successful MD casts
function SimpleMD:Gain()
	local message = string.gsub(self.db.profile.messageGain, "<player>", PlayerName)
	message = string.gsub(message, "<target>", self.SpellTarget)

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		end
		if self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
		if self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		if (IsRaidLeader() or IsRaidOfficer()) and self.db.profile.showRW then
			SendChatMessage(message, "RAID_WARNING")
		end
		
		self:SendMessage(message, 1)
		self:SendCommMessage("GROUP", "GAIN")
		if self.db.profile.showSelf and self.db.profile.showBars then
			self:StartTimer(PlayerName)
		end
	end
end

-- called on Misdirection fading from self
function SimpleMD:Fade()
	local message = string.gsub(self.db.profile.messageFade, "<player>", PlayerName)
	message = string.gsub(message, "<target>", self.SpellTarget)

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		end
		if self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
		if self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		
		self:SendMessage(message, 1)
	end
end

-- test broadcast for making sure colors or channel is correct
function SimpleMD:messageTest()
	if self.db.profile.showInChat then
		self:Print(L["Testing Chat Frame Printing"])
	end
	if self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(L["Testing On Screen Printing"], self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	end
	if self.db.profile.showInChannel and channel ~= "" then
		SendChatMessage(L["Testing Channel Printing"], "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
	end
	if self.db.profile.showFloating then
		self:Pour(L["Testing Floating Message"], self.db.profile.r, self.db.profile.g, self.db.profile.b)
	end
end

function SimpleMD:CreateAnchor(text, r, g, b)
	local f = CreateFrame("BUTTON", nil, UIParent)
	f:SetWidth(200)
	f:SetHeight(25)
	
	f.owner = self
	
	if self.db.profile.position.x and self.db.profile.position.y then
		f:ClearAllPoints()
		f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.position.x, self.db.profile.position.y)
	else
		f:SetPoint("CENTER", UIParent, "CENTER")
	end
	
	f:SetScript("OnDragStart", function() this:StartMoving() end )
	f:SetScript("OnDragStop",
		function ()
			this:StopMovingOrSizing()
			local _, _, _, x, y = this:GetPoint()
			this.owner.db.profile.position.x = math.floor(x)
			this.owner.db.profile.position.y = math.floor(y)
		end)
	
	f:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = false, tileSize = 16, edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
		})
	f:SetBackdropColor(r, g, b, .6)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	
	f.text = f:CreateFontString(nil, "OVERLAY")
	f.text:SetFontObject(GameFontNormalSmall)
	f.text:ClearAllPoints()
	f.text:SetTextColor(1, 1, 1, 1)
	f.text:SetWidth(200)
	f.text:SetHeight(25)
	f.text:SetPoint("TOPLEFT", f, "TOPLEFT")
	f.text:SetJustifyH("CENTER")
	f.text:SetJustifyV("MIDDLE")
	f.text:SetText(text)
	
	f:Hide()
	
	return f
end

function SimpleMD:ToggleAnchor()
	if self.anchor:IsVisible() then
		self.anchor:Hide()
	else
		self.anchor:Show()
	end
end

function SimpleMD:GetTexture(name)
	local i = 1
	while true do
		local spell = GetSpellName(i, BOOKTYPE_SPELL)
		if not spell then break
		elseif spell == name then
			tex = GetSpellTexture(i, BOOKTYPE_SPELL)
			return tex
		end
		i = i + 1
	end
end

function SimpleMD:StartTimer(name)
	local id = name.."1"
	
	for i = 20, 1, -1 do
		if self.bars[i].id == id then
			self:SetCandyBarFade(id, 0, false)
			self:StopCandyBar(id)
			self:StopBar(id)
			break
		end
	end
	
	local slot = 0
	for i = 1, 20 do
		if not self.bars[i].id then 
			slot = i
			break
		end
	end
	
	self.bars[slot].id = id
	self.bars[slot].name = name
	
	local text = name
	local color = self.db.profile.barColor
	
	self:RegisterCandyBar(id, 120, text, self.tex, color, color, color, "red")
	self:RegisterCandyBarWithGroup(id, "SimpleMD")
	if self.db.profile.texture then self:SetCandyBarTexture(id, self.textures[self.db.profile.texture]) end
	if self.db.profile.bg then self:SetCandyBarBackgroundColor(id, self.db.profile.bg, self.db.profile.bg_alpha) end
	if self.db.profile.textColor then self:SetCandyBarTextColor(id, self.db.profile.textColor) end
	if self.db.profile.textSize then self:SetCandyBarFontSize(id, self.db.profile.textSize) end
	if self.db.profile.barScale then self:SetCandyBarScale(id, self.db.profile.barScale) end
	if self.db.profile.barWidth then self:SetCandyBarWidth(id, self.db.profile.barWidth) end
	if self.db.profile.barHeight then self:SetCandyBarHeight(id, self.db.profile.barHeight) end
	self:SetCandyBarReversed(id, self.db.profile.reverse)
	self:SetCandyBarFade(id, 2, true)
	self:SetCandyBarCompletion(id, self.StopBar, self, id)
	self:StartCandyBar(id, true)
end

function SimpleMD:StopBar(id)
	for i = 1, 20 do
		if self.bars[i].id == id then
			self.bars[i] = {}
		end
	end
	
	for i = 1, 19 do
		if not self.bars[i].id then
			local temp = self.bars[i]
			for j = i + 1, 20 do
				if self.bars[j].id then
					self.bars[i] = self.bars[j]; self.bars[j] = temp; temp = nil
					break
				end
			end
			if temp then break end
		end
	end
end

function SimpleMD:DisableBars()
	if self.db.profile.showBars then
		for i = 1, 20 do
			if self.bars[i].id then
				self:StopCandyBar(self.bars[i].id)
			end
		end
	end
	self.db.profile.showBars = not self.db.profile.showBars
end
